import base64
from pypicgo.core.base.result import Result
import requests
from typing import List
from requests import Response
from pypicgo.core.base.uploader import CommonUploader
from pypicgo.core.models import PluginModel
from .schemas import GithubUploaderData
from pypicgo.core.logger import logger


class GithubUploader(CommonUploader):
    name: str = 'githubUploader'
    domain: str = 'https://api.github.com'
    owner: str
    repo: str
    img_path: str
    oauth_token: str

    def __init__(self,
                 owner: str,
                 repo: str,
                 img_path: str,
                 oauth_token: str,
                 plugins: List[PluginModel],
                 **kwargs):

        super().__init__(
            owner=owner,
            repo=repo,
            img_path=img_path,
            oauth_token=oauth_token,
            plugins=plugins,
            **kwargs
        )

    def load_config(self,
                    owner: str,
                    repo: str,
                    img_path: str,
                    oauth_token: str,
                    **kwargs
                    ):
        self.repo = repo
        self.owner = owner
        self.img_path = img_path
        self.oauth_token = oauth_token

        logger.info('load config successfully')

    @property
    def base_url(self):
        return f'{self.domain}/repos/{self.owner}/{self.repo}'

    def _upload_path(self, filename):
        return f'{self.base_url}/contents/{self.img_path}/{filename}'

    def _get_upload_data(self) -> GithubUploaderData:
        with open(self.file.tempfile.resolve(), 'rb') as f:
            base64_data = base64.b64encode(f.read())
            filedata = str(base64_data, encoding='utf-8')
        return GithubUploaderData(
            content=filedata
        )

    def upload(self) -> Result:
        filename = self.file.filename
        data = self._get_upload_data().json()
        headers = {'Authorization': f'token  {self.oauth_token}'}

        resp = requests.put(
            url=self._upload_path(filename),
            headers=headers,
            data=data
        )
        result = self.is_success(resp)
        return result


    def is_success(self, resp: Response) -> Result:
        if resp.status_code == 201:
            download_url = resp.json()['content']['download_url']
            return Result(status=True, file=self.file, message=download_url)
        else:
            reason = resp.json().get('message')
            logger.warning(f'upload fail, message:{reason}')
            return Result(status=False, file=self.file, message=reason)



from base64 import urlsafe_b64encode

_BLOCK_SIZE = 1024 * 1024 * 4


def b(data):
    if isinstance(data, str):
        return data.encode('utf-8')
    return data


def s(data):
    if isinstance(data, bytes):
        data = data.decode('utf-8')
    return data


def urlsafe_base64_encode(data):
    """urlsafe的base64编码:
    对提供的数据进行urlsafe的base64编码。规格参考：
    https://developer.qiniu.com/kodo/manual/1231/appendix#1
    Args:
        data: 待编码的数据，一般为字符串
    Returns:
        编码后的字符串
    """
    ret = urlsafe_b64encode(b(data))
    return s(ret)



from typing import List
from pypicgo.core.base.result import Result
from pypicgo.core.base.file import UploadFile
from pypicgo.core.exceptions import NotImplementedException


class BasePlugin:
    name = 'base_plugin'

    def __init__(self, **kwargs):
        pass


class BeforePlugin(BasePlugin):
    type = 'before'

    def execute(self, file: UploadFile) -> UploadFile:
        raise NotImplementedException()


class AfterPlugin(BasePlugin):
    type = 'after'

    def execute(self, result: Result):
        raise NotImplementedException()


class FinallyPlugin(BasePlugin):
    type = 'final'

    def execute(self, results: List[Result]):
        raise NotImplementedException()



import base64
import requests
import json
from typing import List, Optional, Tuple
from requests import Response
from requests_toolbelt import MultipartEncoder
from pypicgo.core.base.uploader import CommonUploader
from pypicgo.core.models import PluginModel
from pypicgo.core.base.result import Result
from .schemas import GiteeUploaderData
from pypicgo.core.logger import logger


class GiteeUploader(CommonUploader):
    name: str = 'giteeUploader'
    domain: str = 'https://gitee.com'
    owner: str
    repo: str
    img_path: str
    access_token: str

    def __init__(self,
                 owner: str,
                 repo: str,
                 img_path: str,
                 access_token: str,
                 plugins: List[PluginModel],
                 **kwargs
                 ):

        super().__init__(
            owner=owner,
            repo=repo,
            img_path=img_path,
            access_token=access_token,
            plugins=plugins,
            **kwargs
        )

    def load_config(self,
                    owner: str,
                    repo: str,
                    img_path: str,
                    access_token: str,
                    **kwargs
                    ):
        self.repo = repo
        self.owner = owner
        self.img_path = img_path
        self.access_token = access_token

        logger.info('load config successfully')

    @property
    def base_url(self):
        return f'{self.domain}/api/v5/repos/{self.owner}/{self.repo}'

    def _upload_path(self, filename):
        return f'{self.base_url}/contents/{self.img_path}/{filename}'

    def _get_upload_data(self) -> GiteeUploaderData:
        with open(self.file.tempfile.resolve(), 'rb') as f:
            base64_data = base64.b64encode(f.read())
            filedata = str(base64_data, encoding='utf-8')
        return GiteeUploaderData(
            access_token=self.access_token,
            content=filedata
        )

    def upload(self) -> Result:
        filename = self.file.filename
        data = json.loads(self._get_upload_data().json())
        form_data = MultipartEncoder(data)
        headers = {'Content-Type': form_data.content_type}
        resp = requests.post(
            url=self._upload_path(filename),
            headers=headers,
            data=form_data
        )
        result = self.is_success(resp)

        return result

    def is_success(self, resp: Response) -> Result:
        if resp.status_code == 201:
            download_url = resp.json()['content']['download_url']
            return Result(status=True, file=self.file, message=download_url)
        else:
            reason = resp.json().get('message')
            logger.warning(f'upload fail, message:{reason}')
            return Result(status=False, file=self.file, message=reason)



from pathlib import Path
from pypicgo.core.exceptions import (PathNotExistsException, IsNotFileException)
from pypicgo.core.logger import logger


class UploadFile:

    def __init__(self, filepath):
        file = Path(filepath)
        if not file.exists():
            raise PathNotExistsException()
        if not file.is_file():
            raise IsNotFileException()
        self.origin_file = file
        self.filename = file.name
        self._tempfile = file
        logger.info('uploadfile create successfully')

    @property
    def tempfile(self):
        return self._tempfile

    @tempfile.setter
    def tempfile(self, file: Path):
        if isinstance(file, Path):
            if not file.exists():
                raise PathNotExistsException()
            if not file.is_file():
                raise IsNotFileException()
            self._tempfile = file

    def __str__(self):
        return self._tempfile.resolve()



import yaml
from pathlib import Path
from typing import ClassVar, Dict, List
from pypicgo.core.models import ConfigModel, PluginModel
from pypicgo.core.utils.modules import import_string
from pypicgo import BASE_DIR

template = '''
uploader:
  name: pypicgo.uploaders.gitee.uploader.GiteeUploader
  module: 
  config:
plugins:
  - module: pypicgo.plugins.rename.ReNamePlugin
'''


class Settings:
    CONFIG_DIR: Path = BASE_DIR
    uploader_class: ClassVar
    uploader_config: Dict
    plugins: List[PluginModel]

    def __init__(self):
        self.__init_env()
        self.load_config()

    def __init_env(self):
        path = self.CONFIG_DIR.joinpath('config.yaml')
        if not path.exists():
            with path.open(mode='w') as f:
                f.write(template)

    def load_config(self):
        config_file = self.CONFIG_DIR.joinpath('config.yaml')
        with open(config_file.resolve(), 'r', encoding='utf-8') as f:
            config = yaml.load(f.read(), Loader=yaml.CFullLoader)
            model = ConfigModel(uploader=config['uploader'], plugins=config.get('plugins', []))
            try:
                module = model.uploader.module
                self.uploader_class = import_string(module)
            except ImportError:
                raise ImportError(f'uploader {module} doesnt look like a module path')
            self.uploader_config = model.uploader.config
            self.plugins = model.plugins



import os
import platform
from pathlib import Path

system_name = platform.system().lower()

if system_name == 'linux':
    HOME = os.environ.get('HOME')
    home = Path(HOME)
elif system_name == 'windows':
    HOMEDRIVE = os.environ.get('HOMEDRIVE')
    HOME = os.environ.get('HOMEPATH')
    home = Path(HOMEDRIVE).joinpath(HOME)
BASE_DIR = home.joinpath('.PyPicGO')
if not BASE_DIR.exists():
    BASE_DIR.mkdir(parents=True)

__all__ = ["BASE_DIR"]




class BaseException(Exception):
    pass


class NotImplementedException(BaseException):

    def __str__(self):
        return 'Mthod is not implemented'


class NotExtendsException(BaseException):

    def __str__(self):
        return 'base class  is not Extends'


class PathNotExistsException(BaseException):
    def __str__(self):
        return 'then path  is not exists'


class IsNotFileException(BaseException):

    def __str__(self):
        return 'then path is not file'


class UploaderTypeException(BaseException):

    def __str__(self):
        return 'not is UploaderPlugin type'


class PluginExecuteException(BaseException):

    def __str__(self):
        return 'Plugin execute exception'



import sys
import platform
from typing import List
from pypicgo.core.base.plugin import FinallyPlugin
from pypicgo.core.base.result import Result


class TyporaPlugin(FinallyPlugin):
    name = 'Typora'

    def execute(self, results: List[Result]):
        system_name = platform.system().lower()
        messages = []
        for result in results:
            if result.status:
                messages.append(result.message)
        if len(messages) > 0:
            if system_name == 'darwin':
                message = '\r'.join(messages)
                sys.stdout.write(f'Upload Success:\r{message}\r')
            elif system_name == 'linux':
                message = '\n'.join(messages)
                sys.stdout.write(f'Upload Success:\n{message}\n')
            elif system_name == 'windows':
                message = '\r\n'.join(messages)
                sys.stdout.write(f'Upload Success:\r\n{message}\r\n')




from pypicgo.core.base.plugin import AfterPlugin
from .tk import TkinterNotice
from pypicgo.core.base.result import Result


class NotifyPlugin(AfterPlugin):
    name = 'Notify'

    def execute(self, result: Result):
        message = result.message
        if result.status:
            message = f'{result.file.origin_file.name} 已上传成功'
        filepath = result.file.tempfile.resolve()
        TkinterNotice().send(result.status, message, filepath)



from PIL import Image, ImageTk
from tkinter import Tk, Label, LEFT, Canvas, X, Y, TOP
import time
import platform


class TkinterNotice:
    _window = None
    sucess_color = '#67C23A'
    warn_color = '#F56C6C'
    major_color = '#45898A'
    background_color = '#C6E2FF'
    width = 300
    height = 100

    def __init__(self):
        self._window = Tk()
        screen_width = self._window.winfo_screenwidth()
        screen_height = self._window.winfo_screenheight()
        geometry = self._get_geometry(screen_width, screen_height)
        self._window.geometry(geometry)
        self._window.configure(bg=self.background_color)
        self._window.overrideredirect(True)

    def _get_geometry(self, width: int, height: int):
        system_name = platform.system().lower()
        if system_name == 'linux':
            return f'{self.width}x{self.height}+{(width - self.width) // 2}+0'
        elif system_name == 'windows':
            return f'{self.width}x{self.height}+{width - self.width - 10}+{height - self.height - 110}'
        elif system_name == 'darwin':
            return f'{self.width}x{self.height}+{(width - self.width) // 2}+0'
        else:
            return f'{self.width}x{self.height}+{(width - self.width) // 2}+0'

    def send(self, is_success: bool, result: str, filepath: str):
        # 创建550 * 300的画布
        canvas = Canvas(self._window, bg=self.background_color, width=100, height=100)
        # 在画布上创建图像，放置导入图片
        img = Image.open(filepath)
        width, height = img.size
        width = width * self.height // height
        # 以通知窗体的高度为基准，等比例缩放图片生成缩略图
        img.thumbnail((width, self.height), Image.ANTIALIAS)
        small = 100
        # 去出来的部分为中心部分的矩形（如300*100取的是(100,0,200,100)）
        box = ((img.size[0] - small) / 2, (img.size[1] - small), (img.size[0] - small) / 2 + small,
               (img.size[1] - small) / 2 + small)
        # 裁切缩略图片
        img = img.crop(box)
        width = img.size[0]
        # 生成图片数据
        image = ImageTk.PhotoImage(img)

        # 向画布绘图
        canvas.create_image(width // 2, 0, anchor='n', image=image)
        canvas.configure(highlightthickness=0)
        title = 'PyPicGo上传成功'
        msg = f'返回信息为：{result}'
        color = self.sucess_color
        if not is_success:
            color = self.warn_color
            title = 'PyPicGo上传失败'

        label_tiitle = Label(
            master=self._window,
            text=title,
            wraplength=240,
            bg=self.background_color,
            font=(None, 15),
            fg=color)
        label_body = Label(
            master=self._window,
            text=msg,
            wraplength=240,
            bg=self.background_color,
            fg=self.major_color,
            )
        canvas.pack(fill=Y, side=LEFT)  # #相对布局
        label_tiitle.pack(fill=X, side=TOP)
        label_body.pack()

        self._window.after(1000, self.destroy)
        self._window.mainloop()

    def destroy(self):
        for i in range(100, 0, 5):  # 这里不会生成775，要使用比750大25的数字结尾
            self._window.attributes('-alpha', i / 100.0)
            time.sleep(0.01)

        self._window.after(1, self._window.destroy)



from pydantic import BaseModel
from typing import Dict, Optional, List


class PluginModel(BaseModel):
    module: str
    config: Optional[Dict] = None


class UploaderModel(BaseModel):
    name: str
    module: str
    config: Optional[Dict] = None


class ConfigModel(BaseModel):
    uploader: UploaderModel
    plugins: List[PluginModel]



import sys
from pypicgo.core.config import Settings
from pypicgo.core.execute import create_uploader
from pypicgo.core.logger import logger


def action():
    argv = sys.argv
    if len(argv) == 1:
        logger.info('PyPicGo has been installed successfully')
    elif len(argv) > 1:
        settings = Settings()
        uploader = settings.uploader_class
        uploader_config = settings.uploader_config
        plugins = settings.plugins
        with create_uploader(uploader, uploader_config, plugins) as uploader:
            files = argv[1:]
            for filepath in files:
                uploader.do(filepath)


if __name__ == '__main__':
    action()



import base64
import binascii
from typing import Optional, List
from pydantic import BaseModel, validator, ValidationError
from pypicgo.core.models import PluginModel


class GithubUploaderData(BaseModel):
    message: str = 'pyPicGo upload success'
    content: str

    @validator('content')
    def validate_content(cls, content):
        try:
            if base64.b64encode(base64.b64decode(content.encode('utf-8'))) == content.encode('utf-8'):
                return content
            else:
                raise ValidationError('Non-base64 digit found')
        except binascii.Error('Non-base64 digit found'):
            raise ValidationError('Non-base64 digit found')


class GithubUploaderConfig(BaseModel):
    owner: str
    repo: str
    img_path: str
    oauth_token: str
    plugins: Optional[List[PluginModel]] = []



from typing import ClassVar, Dict, List
from pypicgo.core.base.uploader import CommonUploader
from pypicgo.core.logger import logger
from pypicgo.core.exceptions import UploaderTypeException
from pypicgo.core.models import PluginModel


class create_uploader:

    def __init__(self, uploader_class: ClassVar[CommonUploader], config: Dict, plugins: List[PluginModel]):
        if issubclass(uploader_class, CommonUploader):
            self.uploader = uploader_class(plugins=plugins, **config)
            logger.info('create uploader successfully')
        else:
            raise UploaderTypeException()

    def __enter__(self):
        self.uploader.get_plugins()
        return self.uploader

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.uploader.final()



import datetime
import hashlib
import re

from pypicgo.core.base.file import UploadFile
from pypicgo.core.base.plugin import BeforePlugin


class ReNamePlugin(BeforePlugin):
    name = 'ReName'

    def __init__(self, format=None):
        self.format = format
        super().__init__()

    def execute(self, file: UploadFile) -> UploadFile:
        suffix = file.tempfile.suffix
        if not self.format:
            return file
        else:
            names = self.parse(self.format)
            func_map = self.get_variable_func()
            for index in range(len(names)):
                name = names[index]
                if name in func_map:
                    names[index] = func_map[name](file)
            filename = ''.join(names)
            file.filename = f'{filename}{suffix}'
            return file

    def _date(self, file: UploadFile) -> str:
        return datetime.datetime.now().strftime("%Y%m%d%H%M%S")

    def _hash(self, file: UploadFile) -> str:
        myhash = hashlib.md5()
        f = open(file.tempfile.resolve(), 'rb')
        while True:
            b = f.read(8096)
            if not b:
                break
            myhash.update(b)
        f.close()
        md5 = myhash.hexdigest()
        return md5

    def _filename(self, file: UploadFile) -> str:
        return file.filename.split('.')[0]

    def _location(self, file: UploadFile) -> str:
        path = str(file.origin_file.resolve())
        filename = file.origin_file.name
        location = path.split(filename)[0]
        return location

    def get_variable_func(self):
        func_map = {
            'location': self._location,
            'filename': self._filename,
            'hash': self._hash,
            'date': self._date
        }
        return func_map

    def parse(self, format: str):
        # variables = re.findall(r'[{](.*?)[}]', format)
        names = re.split(r'[{}]', format)
        return names
